/*
 * Copyright 2008-2009 Diego Iastrubni, elcuco, at, kde.org
 * Copyright 2008-2009 Benjamin C. Meyer <ben@meyerhome.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Benjamin Meyer nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef LANGUAGEMANAGER_H
#define LANGUAGEMANAGER_H

#include <qobject.h>

#include <qstringlist.h>

class QTranslator;
class LanguageManager : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString currentLanguage READ currentLanguage WRITE setCurrentLanguage)

signals:
    void languageChanged(const QString &language);

public:
    LanguageManager(QObject *parent = 0);

    void addLocaleDirectory(const QString &directory);
    QStringList localeDirectories() const;

    QString currentLanguage() const;
    bool setCurrentLanguage(const QString &language);

    QStringList languages() const;
    bool isLanguageAvailable(const QString &language) const;

public slots:
    void loadLanguageFromSettings();
    void chooseNewLanguage();

private:
    QString convertStringToLanguageFile(const QString &string) const;
    void loadAvailableLanguages() const;

    QString m_currentLanguage;
    QTranslator *m_sysTranslator;
    QTranslator *m_appTranslator;
    mutable bool m_loaded;
    mutable QStringList m_languages;
    QStringList m_localeDirectories;
};

#endif //LANGUAGEMANAGER_H

